package java.net;

import sun.misc.SharedSecrets;
import sun.misc.JavaNetHttpCookieAccess;
import java.util.HashMap;
import java.text.SimpleDateFormat;
import java.util.Locale;
import java.util.GregorianCalendar;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Objects;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.TimeZone;
import java.util.Map;

public final class HttpCookie implements Cloneable
{
    private final String name;
    private String value;
    private String comment;
    private String commentURL;
    private boolean toDiscard;
    private String domain;
    private long maxAge;
    private String path;
    private String portlist;
    private boolean secure;
    private boolean httpOnly;
    private int version;
    private final String header;
    private final long whenCreated;
    private static final long MAX_AGE_UNSPECIFIED = -1L;
    private static final String[] COOKIE_DATE_FORMATS;
    private static final String SET_COOKIE = "set-cookie:";
    private static final String SET_COOKIE2 = "set-cookie2:";
    private static final String tspecials = ",; ";
    static final Map<String, HttpCookie.CookieAttributeAssignor> assignors;
    static final TimeZone GMT;
    
    public HttpCookie(String s, String s2) {
        this(s, s2, null);
    }
    
    private HttpCookie(String trim, String value, String header) {
        this.maxAge = -1L;
        this.version = 1;
        trim = trim.trim();
        if (trim.length() == 0 || !isToken(trim) || trim.charAt(0) == '$') {
            throw new IllegalArgumentException("Illegal cookie name");
        }
        this.name = trim;
        this.value = value;
        this.toDiscard = false;
        this.secure = false;
        this.whenCreated = System.currentTimeMillis();
        this.portlist = null;
        this.header = header;
        return;
    }
    
    public static List<HttpCookie> parse(String s) {
        return parse(s, false);
    }
    
    private static List<HttpCookie> parse(String s, boolean b) {
        int guessCookieVersion = guessCookieVersion(s);
        if (startsWithIgnoreCase(s, "set-cookie2:")) {
            s = s.substring("set-cookie2:".length());
        }
        else if (startsWithIgnoreCase(s, "set-cookie:")) {
            s = s.substring("set-cookie:".length());
        }
        ArrayList o = new ArrayList();
        if (guessCookieVersion == 0) {
            HttpCookie internal2 = parseInternal(s, b);
            internal2.setVersion(0);
            o.add(internal2);
            return o;
        }
        Iterator<String> iterator = splitMultiCookies(s).iterator();
        while (iterator.hasNext()) {
            HttpCookie internal = parseInternal((String)iterator.next(), b);
            internal.setVersion(1);
            o.add(internal);
        }
        return o;
    }
    
    public boolean hasExpired() {
        return this.maxAge == 0L || (this.maxAge != -1L && (System.currentTimeMillis() - this.whenCreated) / 1000L > this.maxAge);
    }
    
    public void setComment(String comment) {
        this.comment = comment;
    }
    
    public String getComment() {
        return this.comment;
    }
    
    public void setCommentURL(String commentURL) {
        this.commentURL = commentURL;
    }
    
    public String getCommentURL() {
        return this.commentURL;
    }
    
    public void setDiscard(boolean toDiscard) {
        this.toDiscard = toDiscard;
    }
    
    public boolean getDiscard() {
        return this.toDiscard;
    }
    
    public void setPortlist(String portlist) {
        this.portlist = portlist;
    }
    
    public String getPortlist() {
        return this.portlist;
    }
    
    public void setDomain(String domain) {
        if (domain != null) {
            this.domain = domain.toLowerCase();
        }
        else {
            this.domain = domain;
        }
    }
    
    public String getDomain() {
        return this.domain;
    }
    
    public void setMaxAge(long maxAge) {
        this.maxAge = maxAge;
    }
    
    public long getMaxAge() {
        return this.maxAge;
    }
    
    public void setPath(String path) {
        this.path = path;
    }
    
    public String getPath() {
        return this.path;
    }
    
    public void setSecure(boolean secure) {
        this.secure = secure;
    }
    
    public boolean getSecure() {
        return this.secure;
    }
    
    public String getName() {
        return this.name;
    }
    
    public void setValue(String value) {
        this.value = value;
    }
    
    public String getValue() {
        return this.value;
    }
    
    public int getVersion() {
        return this.version;
    }
    
    public void setVersion(int version) {
        if (version != 0 && version != 1) {
            throw new IllegalArgumentException("cookie version should be 0 or 1");
        }
        this.version = version;
        return;
    }
    
    public boolean isHttpOnly() {
        return this.httpOnly;
    }
    
    public void setHttpOnly(boolean httpOnly) {
        this.httpOnly = httpOnly;
    }
    
    public static boolean domainMatches(String anotherString, String str) {
        if (anotherString == null || str == null) {
            return false;
        }
        int equalsIgnoreCase = ".local".equalsIgnoreCase(anotherString);
        int n = anotherString.indexOf(46);
        if (n == 0) {
            n = anotherString.indexOf(46, 1);
        }
        if (equalsIgnoreCase == 0 && (n == -1 || n == anotherString.length() - 1)) {
            return false;
        }
        if (str.indexOf(46) == -1 && (equalsIgnoreCase || anotherString.equalsIgnoreCase(str + ".local"))) {
            return true;
        }
        int n2 = str.length() - anotherString.length();
        if (n2 == 0) {
            return str.equalsIgnoreCase(anotherString);
        }
        if (n2 > 0) {
            String substring = str.substring(0, n2);
            String substring2 = str.substring(n2);
            return substring.indexOf(46) == -1 && substring2.equalsIgnoreCase(anotherString);
        }
        return n2 == -1 && anotherString.charAt(0) == '.' && str.equalsIgnoreCase(anotherString.substring(1));
    }
    
    @Override
    public String toString() {
        if (this.getVersion() > 0) {
            return this.toRFC2965HeaderString();
        }
        return this.toNetscapeHeaderString();
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof HttpCookie)) {
            return false;
        }
        HttpCookie httpCookie = (HttpCookie)o;
        return equalsIgnoreCase(this.getName(), httpCookie.getName()) && equalsIgnoreCase(this.getDomain(), httpCookie.getDomain()) && Objects.equals(this.getPath(), httpCookie.getPath());
    }
    
    @Override
    public int hashCode() {
        return this.name.toLowerCase().hashCode() + ((this.domain != null) ? this.domain.toLowerCase().hashCode() : 0) + ((this.path != null) ? this.path.hashCode() : 0);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException ex) {
            throw new RuntimeException(ex.getMessage());
        }
    }
    
    private static boolean isToken(String s) {
        int length = s.length();
        int i = 0;
        while (i < length) {
            int char1 = s.charAt(i);
            if (char1 < ' ' || char1 >= '\u007f' || ",; ".indexOf(char1) != -1) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    private static HttpCookie parseInternal(String str, boolean b) {
        StringTokenizer stringTokenizer = new StringTokenizer(str, ";");
        HttpCookie httpCookie;
        try {
            String nextToken = stringTokenizer.nextToken();
            int index = nextToken.indexOf(61);
            if (index != -1) {
                String trim = nextToken.substring(0, index).trim();
                String trim2 = nextToken.substring(index + 1).trim();
                if (b) {
                    httpCookie = new HttpCookie(trim, stripOffSurroundingQuote(trim2), str);
                }
                else {
                    httpCookie = new HttpCookie(trim, stripOffSurroundingQuote(trim2));
                }
            }
            else {
                throw new IllegalArgumentException("Invalid cookie name-value pair");
            }
        }
        catch (NoSuchElementException ex) {
            throw new IllegalArgumentException("Empty cookie header string");
        }
        while (stringTokenizer.hasMoreTokens()) {
            String nextToken2 = stringTokenizer.nextToken();
            int index2 = nextToken2.indexOf(61);
            String s;
            String trim3;
            if (index2 != -1) {
                s = nextToken2.substring(0, index2).trim();
                trim3 = nextToken2.substring(index2 + 1).trim();
            }
            else {
                s = nextToken2.trim();
                trim3 = null;
            }
            assignAttribute(httpCookie, s, trim3);
        }
        return httpCookie;
    }
    
    private static void assignAttribute(HttpCookie httpCookie, String s, String stripOffSurroundingQuote) {
        stripOffSurroundingQuote = stripOffSurroundingQuote(stripOffSurroundingQuote);
        HttpCookie.CookieAttributeAssignor cookieAttributeAssignor = (HttpCookie.CookieAttributeAssignor)HttpCookie.assignors.get(s.toLowerCase());
        if (cookieAttributeAssignor != null) {
            cookieAttributeAssignor.assign(httpCookie, s, stripOffSurroundingQuote);
        }
    }
    
    private String header() {
        return this.header;
    }
    
    private String toNetscapeHeaderString() {
        return this.getName() + "=" + this.getValue();
    }
    
    private String toRFC2965HeaderString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.getName()).append("=\"").append(this.getValue()).append('\"');
        if (this.getPath() != null) {
            sb.append(";$Path=\"").append(this.getPath()).append('\"');
        }
        if (this.getDomain() != null) {
            sb.append(";$Domain=\"").append(this.getDomain()).append('\"');
        }
        if (this.getPortlist() != null) {
            sb.append(";$Port=\"").append(this.getPortlist()).append('\"');
        }
        return sb.toString();
    }
    
    private long expiryDate2DeltaSeconds(String source) {
        GregorianCalendar gregorianCalendar = new GregorianCalendar(HttpCookie.GMT);
        int i = 0;
        while (i < HttpCookie.COOKIE_DATE_FORMATS.length) {
            SimpleDateFormat o = new SimpleDateFormat(HttpCookie.COOKIE_DATE_FORMATS[i], Locale.US);
            gregorianCalendar.set(1970, 0, 1, 0, 0, 0);
            o.setTimeZone(HttpCookie.GMT);
            o.setLenient(false);
            o.set2DigitYearStart(gregorianCalendar.getTime());
            try {
                gregorianCalendar.setTime(o.parse(source));
                if (!HttpCookie.COOKIE_DATE_FORMATS[i].contains("yyyy")) {
                    int value = gregorianCalendar.get(1) % 100;
                    if (value < 70) {
                        value += 2000;
                    }
                    else {
                        value += 1900;
                    }
                    gregorianCalendar.set(1, value);
                }
                return (gregorianCalendar.getTimeInMillis() - this.whenCreated) / 1000L;
            }
            catch (Exception ex) {
                ++i;
            }
        }
        return 0L;
    }
    
    private static int guessCookieVersion(String lowerCase) {
        int n = 0;
        lowerCase = lowerCase.toLowerCase();
        if (lowerCase.indexOf("expires=") != -1) {
            n = 0;
        }
        else if (lowerCase.indexOf("version=") != -1) {
            n = 1;
        }
        else if (lowerCase.indexOf("max-age") != -1) {
            n = 1;
        }
        else if (startsWithIgnoreCase(lowerCase, "set-cookie2:")) {
            n = 1;
        }
        return n;
    }
    
    private static String stripOffSurroundingQuote(String s) {
        if (s != null && s.length() > 2 && s.charAt(0) == '\"' && s.charAt(s.length() - 1) == '\"') {
            return s.substring(1, s.length() - 1);
        }
        if (s != null && s.length() > 2 && s.charAt(0) == '\'' && s.charAt(s.length() - 1) == '\'') {
            return s.substring(1, s.length() - 1);
        }
        return s;
    }
    
    private static boolean equalsIgnoreCase(String s, String anotherString) {
        return s == anotherString || (s != null && anotherString != null && s.equalsIgnoreCase(anotherString));
    }
    
    private static boolean startsWithIgnoreCase(String s, String s2) {
        return s != null && s2 != null && (s.length() >= s2.length() && s2.equalsIgnoreCase(s.substring(0, s2.length())));
    }
    
    private static List<String> splitMultiCookies(String s) {
        ArrayList<String> list = new ArrayList();
        int n = 0;
        int i = 0;
        int n2 = 0;
        while (i < s.length()) {
            int char1 = s.charAt(i);
            if (char1 == '\"') {
                ++n;
            }
            if (char1 == ',' && n % 2 == 0) {
                list.add(s.substring(n2, i));
                n2 = i + 1;
            }
            ++i;
        }
        list.add(s.substring(n2));
        return list;
    }
    
    static long access$000(HttpCookie httpCookie, String s) {
        return httpCookie.expiryDate2DeltaSeconds(s);
    }
    
    static List access$100(String s, boolean b) {
        return parse(s, b);
    }
    
    static {
        COOKIE_DATE_FORMATS = new String[] { "EEE',' dd-MMM-yyyy HH:mm:ss 'GMT'", "EEE',' dd MMM yyyy HH:mm:ss 'GMT'", "EEE MMM dd yyyy HH:mm:ss 'GMT'Z", "EEE',' dd-MMM-yy HH:mm:ss 'GMT'", "EEE',' dd MMM yy HH:mm:ss 'GMT'", "EEE MMM dd yy HH:mm:ss 'GMT'Z" };
        (assignors = new HashMap()).put("comment", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String comment) {
                if (httpCookie.getComment() == null) {
                    httpCookie.setComment(comment);
                }
            }
        });
        HttpCookie.assignors.put("commenturl", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String commentURL) {
                if (httpCookie.getCommentURL() == null) {
                    httpCookie.setCommentURL(commentURL);
                }
            }
        });
        HttpCookie.assignors.put("discard", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                httpCookie.setDiscard(true);
            }
        });
        HttpCookie.assignors.put("domain", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String domain) {
                if (httpCookie.getDomain() == null) {
                    httpCookie.setDomain(domain);
                }
            }
        });
        HttpCookie.assignors.put("max-age", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                try {
                    long long1 = Long.parseLong(s2);
                    if (httpCookie.getMaxAge() == -1L) {
                        httpCookie.setMaxAge(long1);
                    }
                }
                catch (NumberFormatException ex) {
                    throw new IllegalArgumentException("Illegal cookie max-age attribute");
                }
            }
        });
        HttpCookie.assignors.put("path", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String path) {
                if (httpCookie.getPath() == null) {
                    httpCookie.setPath(path);
                }
            }
        });
        HttpCookie.assignors.put("port", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                if (httpCookie.getPortlist() == null) {
                    httpCookie.setPortlist((s2 == null) ? "" : s2);
                }
            }
        });
        HttpCookie.assignors.put("secure", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                httpCookie.setSecure(true);
            }
        });
        HttpCookie.assignors.put("httponly", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                httpCookie.setHttpOnly(true);
            }
        });
        HttpCookie.assignors.put("version", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                try {
                    httpCookie.setVersion(Integer.parseInt(s2));
                }
                catch (NumberFormatException ex) {}
            }
        });
        HttpCookie.assignors.put("expires", new HttpCookie.CookieAttributeAssignor() {
            @Override
            public void assign(HttpCookie httpCookie, String s, String s2) {
                if (httpCookie.getMaxAge() == -1L) {
                    httpCookie.setMaxAge(HttpCookie.access$000(httpCookie, s2));
                }
            }
        });
        SharedSecrets.setJavaNetHttpCookieAccess(new JavaNetHttpCookieAccess() {
            public List<HttpCookie> parse(String s) {
                return HttpCookie.access$100(s, true);
            }
            
            public String header(HttpCookie httpCookie) {
                return httpCookie.header;
            }
        });
        GMT = TimeZone.getTimeZone("GMT");
    }
    
    interface CookieAttributeAssignor
    {
        public abstract void assign(HttpCookie p0, String p1, String p2);
    }
}
